package command;

import java.io.*;
import java.lang.reflect.*;

public class MediaFactory
{
	final String methodName = "getInstance";
	Class[] formalParams = new Class[1];

	public MediaFactory()
	{
		try
		{
			formalParams[0] = Class.forName("java.io.BufferedReader");
		}
		catch(ClassNotFoundException cnfe)
		{
			cnfe.printStackTrace();
			throw new IncompleteDataException("Cannot find java.io.BufferedReader class");
		}
	}

	public Media nextMedia(BufferedReader br) throws IncompleteDataException
	{
		String className = null;
		try
		{
			className = br.readLine();
			if(className == null)
			{
				return null;
			}
			Class clazz = Class.forName(className);
			Method method = clazz.getMethod(methodName, formalParams);
			Object[] actParams = new Object[1];
			actParams[0] = br;
			return (Media)method.invoke(null, actParams);
		}
		catch(InvocationTargetException ita)
		{
			ita.printStackTrace();
			IncompleteDataException e = new IncompleteDataException(methodName + "method for media type " + className + " threw an exception");
			e.initCause(ita);
			throw e;
		}
		catch(IllegalAccessException iae)
		{
			iae.printStackTrace();
			throw new IncompleteDataException("Cannot invoke " + methodName + "method for media type " + className);
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
			throw new IncompleteDataException("Cannot read media type " + className);
		}
		catch(ClassNotFoundException cnfe)
		{
			cnfe.printStackTrace();
			throw new IncompleteDataException("Cannot find media class " + className);
		}
		catch(NoSuchMethodException nsme)
		{
			nsme.printStackTrace();
			throw new IncompleteDataException("Cannot find method " + methodName + " for media type " + className);
		}
	}
}
