package factory;

import java.util.*;

public class ArrayList<E>
extends AbstractList<E>
{
	E[] storage;
	int head, tail, len;

	public ArrayList(int startLen)
	{
		head = tail = len = 0;
		storage = (E[])(new Object[startLen]);
	}

	public ArrayList()
	{
		this(50);
	}

	/**
	* metodi necessari a rendere concreta la classe.
	*/
	List<E> getConcreteInstance()
	{
		return new ArrayList();
	}

	public ListIterator<E> listIterator(int index)
	{
		return new ArrayListIterator<E>(this, index);
	}

	/**
	* Metodi sovrascritti per maggiore efficienza.
	*/

	public int size()
	{
		return len;
	}

	/**
	* Classe iteratore.
	*/
	class ArrayListIterator<E> 
	implements ListIterator<E>
	{
		private ArrayList<E> list;
		int last = 0;//-1 se ultima operazione e' previous, 1 se next, 0 se nessuna
		int cursor; // posizione relativa alla head della lista dell'elemento
								// ritornato dalla prossima chiamata a next().

		ArrayListIterator(ArrayList ref, int index)
		{
			if(index < 0 || index > ref.len)
			{
				throw new IndexOutOfBoundsException("index = " + index);
			}
			list = ref;
			cursor = index;

			System.out.println("Cursore: " + cursor + "; index = " + index + "; head = " + list.head + "; storage.length = " + list.storage.length);
		}

		ArrayListIterator(ArrayList ref)
		{
			this(ref, 0);
		}

		public boolean hasNext()
		{
			return (cursor < list.len);
		}

		public int nextIndex()
		{
			return cursor;
		}

		public boolean hasPrevious()
		{
			return (cursor > 0);
		}

		public int previousIndex()
		{
			return (cursor-1);
		}

		public E next()
		{
			if(!hasNext())
			{
				throw new NoSuchElementException("next in posizione " + cursor);
			}
			E retval = list.storage[(cursor+list.head)%list.storage.length];
			cursor++;
			last = 1;
			return retval;
		}

		public E previous()
		{
			if(!hasPrevious())
			{
				throw new NoSuchElementException("previous in posizione " + cursor);
			}
			E retval = list.storage[(cursor+list.head-1)%list.storage.length];
			last = -1;
			cursor--;
			return retval;
		}

		public void add(E element)
		{
			if(list.storage.length == list.len)
			{
				// ho riempito l'array, ne devo istanziare una nuova grossa il
				// doppio e copiare tutto.
				E[] oldStorage = list.storage;
				list.storage = (E[])(new Object[oldStorage.length*2]);
				int i = 0;
				// Copio tutti gli elementi fino alla posizione i
				for(i=0;i<cursor;i++)
				{
					list.storage[i] = oldStorage[(i+list.head)%oldStorage.length];
				}
				// Inserisco l'oggetto in posizione i
				list.storage[i] = element;
				// Copio gli altri elementi
				for(;i<list.len;i++)
				{
					list.storage[i+1] = oldStorage[(i+list.head)%oldStorage.length];
				}
				list.head = 0;
				list.tail = (++list.len);
			}
			else // ci stiamo
			{
				int i = 0;
				for(i=list.len;i>=cursor;i--)
				{
					list.storage[(i+1+list.head)%list.storage.length] = list.storage[(i+list.head)%list.storage.length];
				}
				list.storage[(cursor+list.head)%list.storage.length] = element;
			}
			cursor++;
			list.tail = (++list.len)%list.storage.length;
			last = 0;
		}

		public void remove()
		{
			switch(last)
			{
				case 0: // nessuno dei due
					throw new IllegalStateException();
					break;
				case 1://last next()
					// C' un unico elemento
					if(size()==1){
						storage = null;
						list.head = list.tail = 0;
						len = 0;
					}
					// Sono alla fine
					else if(cursor == list.len){
						storage[cursor] = null;
						// storage[(cursor+list.head)%list.storage.length] = null;
						list.tail--;
						len--;
					}
					// Ce n' pi di uno
					else{
						storage[cursor] = null;
						int i;
						// ricompatto
						for(i=cursor;i<list.len;i++){
							storage[i]=storage[i+1];
						}
						// ultimo elemento = null
						storage[list.len]=null;
						list.tail--;
						len--;
					}
					break;
				case -1: // previous
					// Un elemento
					if(len==1){
						storage = null;
						list.head = list.tail = 0;
						list.len = 0;
					}
					/*	
					// Sono all'inizio
					else if(cursor == 0){
						
					}
					*/
					// In mezzo
					else{
						storage[cursor] = null;
						int i;
						// ricompatto
						for(i=cursor;i<list.len;i++){
							storage[i]=storage[i+1];
						}
						// ultimo elemento = null
						storage[list.len]=null;
						list.tail--;
						len--;
					}
					break;
				default:
					System.err.println("last ha un valore impossibile: " + last);
					throw new RuntimeException("last ha un valore impossibile: " + last);
			}
		}
	}
}
